
	include regs.inc

	section .init      ; absolute 2:4000, reset vector copy
vec
	dc.w 0002h         ; 00 0 ZK SK PK   ; LSnybble used directly by loader
	dc.w _reboot       ; 02 PC           ; used directly by loader
	dc.w 0             ; 04 SP
	dc.w 0             ; 06 IZ
	dc.w trap_bkpt     ; 08 V_BKPT
	dc.w trap_berr     ; 0A V_BERR
	dc.w trap_swi      ; 0C V_SWI
	dc.w trap_ill      ; 0E V_ILL
	dc.w trap_zdiv     ; 10 V_ZDIV
	dc.w trap          ; 12
	dc.w trap          ; 14
	dc.w trap          ; 16
	dc.w trap          ; 18
	dc.w trap          ; 1A
	dc.w trap          ; 1C
	dc.w trap_uninit   ; 1E V_UNINIT
	dc.w trap          ; 20
	dc.w trap_l1av     ; 22 V_L1AV
	dc.w trap_l2av     ; 24 V_L2AV
	dc.w trap_l3av     ; 26 V_L3AV
	dc.w trap_l4av     ; 28 V_L4AV
	dc.w trap_l5av     ; 2A V_L5AV
	dc.w trap_l6av     ; 2C V_L6AV
	dc.w trap_l7av     ; 2E V_L7AV
	dc.w trap_spur     ; 30 V_SPUR
evec

	global _reboot

_reboot
	orp  # 00F0h       ; also set dsp saturation mode, just for fun    

	ldab # FF
	tbek               ; EXT internal RAM and registers

	ldab # 0Ch
	stab SYPCR         ; system protection control = 0C

	ldd  # 00CFh
	std  SIMCR         ; SIM configuration

	ldd  # 68B0h
	std  CSORBT        ; chip-select boot option = 68B0

	; port pins

	clr  PORTC

	clr  PEPAR         ; PORTE all gpio
	clr  DDRE          ; all input
	clr  PORTE         ; just in case

	clr  PFPAR         ; PORTF all gpio
	ldab # ~1          ; MODCLK pin is input
	stab DDRF
	ldab # 0C0h
	stab PORTF         ; lowest tx_power, rest 0

	ldab # ~2          ; RTS2 is input
	stab DDRGP
	ldab # ~3
	stab PORTGP

	ldab # 8Eh         ; TXD,PCS0,SCK,MOSI output. RTSA,DTRA,DTRB,MISO input
	stab DDRQS
	ldab # 07h         ; all but SCK,MOSI,MISO are gpio
	stab QSPAR
	clr  PORTQS

	; pit

	ldd  # 0251h ; PIRQL 2 PIV 51 XXX 52 later
	std  PICR
	ldd  # 0008h ; 1024 Hz
	std  PITR

	; gpt and pwm

	ldd  # 000Eh
	std  GPTMCR
	ldd  # 9640h
	std  ICR
	clrw TCTL1    ; and TCL2, bytes
	ldd  # 1002h  ; OC2 enable, CPR=2 meaning sysclk/16
	std  TMSK1    ; and TMSK2, bytes
	clrw CFORC    ; and PWMC, bytes
	clrw PWMA     ; and PWMB, bytes

	ldd  # -1
	subd PWMCNT
	lsrd
	lsrd
	lsrd
	andd # 03FFh
	addd TCNT
	std  TOC2
	bclr TFLG1, # 10h  ; OC2F

	; sci and spi

	ldd  # 0450h
	std  QILR       ; and QIVR, bytes
	ldd  # 0084h
	std  QMCR
	ldd  # 001Bh    ; 19200bd
	std  SCCR0
	ldd  # 002Ch    ; interrupts, tx and rx enable
	std  SCCR1
	ldd  SCSR

	; adc

	clrw ADCMCR
	ldd  # 008Ch    ; 10-bit, 2 ADC clock periods, prescaler 0xC
	std  ADCTL0
	ldd  # 0070h    ; scan, multiple, 8
	std  ADCTL1
	ldd  ADCSTAT

	; exceptions and interrupts

	tpd              ; .init
	tbxk

	clrb             ; .data
	tbyk
	ldy  # 0         ; 0:0000 real vectors

	ldx  # vec       ; first few tabularised
L0
	ldd  0,x
	std  0,y
	aix  # 2
	aiy  # 2
	cpx  # evec
	blo  L0

	ldd  # trap      ; any other exception
L1
	std  0,y
	aiy  # 2
	cpy  # 0070h     ; first user interrupt
	blo  L1

	ldd  # interrupt ; any user interrupt. fixed later as needed
L2
	std  0,y
	aiy  # 2
	cpy  # 0200h     ; end of vectors
	blo  L2

	; copy initialized data, clear rest of ram

	ldx # __data_image    ; stash of initialized data here
L3
	ldab 0,x
	stab 0,y
	aix  # 1
	aiy  # 1
	cpy  # __edata
	blo  L3
L4
	clr  0,y
	aiy  # 1
	bne  L4               ; while we are at it, zap all of ram (yk++)

	ldab # FF             ; clear internal RAM too
	tbyk                  ; and iy still 0 from above
L5
	clr  0,y
	aiy  # 1
	cpy  # 0400h          ; 1024 bytes of ram at F:0
	blo  L5

	; "C" segments and stack

	clrb
	tbek
	tbxk
	tbyk
	tbzk                  ; only place
	tbsk                  ; only place
	lds  # __stack
	ldz  # 0

	; reboot(main())

	jsr  _main            ; still ipl 7
	tde
	jmp  _reboot


TE     equ 0008h   ; transmitter enable
RE     equ 0004h   ; transmitter enable
TDRE_  equ 01h     ; transmit data register empty
SCSR_  equ SCSR+1  ; LSByte
TC     equ 80h     ; transmit complete
SCDR_  equ SCDR+1  ; LSByte

puthex_byte
	tba            ; lower nibble saved in A
	lsrb
	lsrb
	lsrb
	lsrb
	bsr  puthex    ; upper nibble
	tab            ; lower nibble back
puthex
	andb # 15
	cmpb # 10
	bcs  * + 4
	addb # 7
	addb # '0'
putc
	brclr SCSR, # TDRE_, * ; XXX can get stuck if brg has been fouled
	stab  SCDR_
	rts


panic                   ; ix points to message in this segment
	orp  # 00E0h

	ldab # FF
	tbek

	bset PORTF, # 0C0h ; lowest tx_power

	ldd  # TE + RE
	std  SCCR1          ; enable async tx (and rx, why ?)

	pulm e              ; get trap PK
	pulm y              ; get trap PC

	clrb
	tbsk
	lds  # __stack      ; safe stack now

	tpd
	tbxk                ; to access the message

	ldab # 0Ah          ; print newline
	bsr  putc
	ldab # 0Dh
	bsr  putc
L6
	ldab 0,x
	bsr  putc
	aix  #+1
	tst  0,x
	bne  L6             ; keep printing message

	ldab # 20h
	bsr  putc           ; blanks are always nice

	xgde                ; trap PK into B
	bsr  puthex         ; just 4 LSbits

	xgey                ; trap PC into E
	ted                 ; MSByte ...
	xgab                ; ... into B
	bsr  puthex_byte
	ted                 ; LSByte into B
	bsr  puthex_byte

	brclr SCSR_, # TC, * ; drain output

	jmp  _reboot

msg_exception   dc.b 'UNKNOWN EXCEPTION'
                dc.b 0
msg_interrupt   dc.b 'UNKNOWN INTERRUPT'
                dc.b 0
msg_bkpt        dc.b 'BREAKPOINT'
                dc.b 0
msg_berr        dc.b 'BUS ERROR'
                dc.b 0
msg_swi         dc.b 'SOFTWARE INTERRUPT'
                dc.b 0
msg_ill         dc.b 'ILLEGAL INSTRUCTION'
                dc.b 0
msg_zdiv        dc.b 'DIVISION BY ZERO'
                dc.b 0
msg_uninit      dc.b 'UNINITIALIZED INTERRUPT'
				dc.b 0
msg_l1av        dc.b 'LEVEL1 AVEC'
                dc.b 0
msg_l2av        dc.b 'LEVEL2 AVEC'
                dc.b 0
msg_l3av        dc.b 'LEVEL3 AVEC'
                dc.b 0
msg_l4av        dc.b 'LEVEL4 AVEC'
                dc.b 0
msg_l5av        dc.b 'LEVEL5 AVEC'
                dc.b 0
msg_l6av        dc.b 'LEVEL6 AVEC'
                dc.b 0
msg_l7av        dc.b 'LEVEL7 AVEC'
                dc.b 0
msg_spur        dc.b 'SPURIOUS INTERRUPT'
                dc.b 0

msg_stack       dc.b 'STACK UNDERFLOW'    ; used programmatically
                dc.b 0

	align 1


	global stack_underflow  ; called manually by stack checker

stack_underflow
	ldx  # msg_stack
	jmp  panic


	global _halt

_halt
	clre
	jsr  _POWEL_pin     ; deactivate power relay (optional)

	lde  # 5
	jsr  _delay_ms      ; wait relay, if any, to release

	pshm k,ccr          ; still here, no relay.
	orp  # 000E0h       ; IPL 7

	lde  # 0FFh         ; supplies off ...
	jsr  _radio_control ; ... from radio module (POWEL reactivated)

halted
	clrb
	tbek                ; EXT RAM

PIT_V equ 0A4h  ; V 52h

	ldd  # just_reti
	std  PIT_V

	ldab # FF
	tbek                ; EXT FF

	ldd  # 0752h        ; IPL 7 IV 52h
	std  PICR
	ldd  # 0100h + 160  ; 10 seconds (PTP "1" and PITM)
	std  PITR

	lde  # 27F1h        ; LPSTOP
	ldd  # 27F7h        ; RTS
	sted _lpstop

	orp  # 000E0h       ; IPL 7
	andp # 07FDFh       ; CCR.S clear, enables LPSTOP; IPL 6

	bclrw SYNCR, # 3    ; STSIM and STEXT off, stop vco and clkout
	inc  PORTC
	bclr PORTC, # 0FEh  ; other leds off

	jsr  _lpstop

	bra  halted

	pulm k,ccr          ; doh ? still here ?
	clre
	jmp  _reboot

	section .bankf
	align 1

_lpstop ds.w 2          ; enter LPSTOP with ADDR18 "1"

	;------------------------------------
	section .text
	align 1

	global text_start

text_start

	;------------------------------------
	section .inthand
	align 1

just_reti
	rti

trap
	ldx  # msg_exception
jump
	jmp  panic

interrupt
	ldx  # msg_interrupt
	bra  jump

trap_bkpt
	ldx  # msg_bkpt
	bra  jump
trap_berr
	ldx  # msg_berr
	bra  jump
trap_swi
	ldx  # msg_swi
	bra  jump
trap_ill
	ldx  # msg_ill
	bra  jump
trap_zdiv
	ldx  # msg_zdiv
	bra  jump
trap_uninit
	ldx  # msg_uninit
	bra  jump
trap_l1av
	ldx  # msg_l1av
	bra  jump
trap_l2av
	ldx  # msg_l2av
	bra  jump
trap_l3av
	ldx  # msg_l3av
	bra  jump
trap_l4av
	ldx  # msg_l4av
	bra  jump
trap_l5av
	ldx  # msg_l5av
	bra  jump
trap_l6av
	ldx  # msg_l6av
	bra  jump
trap_l7av
	ldx  # msg_l7av
	bra  jump
trap_spur
	ldx  # msg_spur
	bra  jump

	end

