/*
 * Looking from component side, cpu on the right,
 *
 *           | |  ||||||||
 *           + G   + TR
 *           3 n   3 xx
 *             d
 * or,
 *           |    ||||||||||    ||
 *           +    TGR+
 *           5    xnx5
 *                 d
 *
 *  Power on
 *   AT           aaaaalo ?
 *   AT+CGMM      request model identification
 *   AT+CGMR      request revision identification
 *  WWW key
 *   AT*EDME?     are data menus enabled?
 *   AT*ESVM?     current voice mail server number?
 *   AT*ESVM=0
 *   AT+CKPD="eseee<see<<s>>ss1"
 *   AT+CKPD="c",25
 *   AT+CKPD="91c91c911111110c"
 *   AT*ESVM=0
 *  Attachment key
 *   AT+CKPD="#11111110c"
 *  Email key
 *   AT*EDME?     are data menus enabled?
 *   AT*ESVM?     current voice mail server number?
 *   AT*ESVM=0
 *   AT+CKPD="eseee<see<<s>>ss1"
 *   AT+CKPD="c",25
 *   AT+CKPD="8666*1111111*0c"
 *   AT*ESVM=0
 *  SMS key
 *   AT*EDME?     are data menus enabled?
 *   AT*ESVM?     current voice mail server number?
 *   AT*ESVM=0
 *   AT+CKPD="eseee<see<<s>>ss1"
 *   AT+CKPD="c",25
 *   AT*ESVM=0
 *  Phonebook key
 *   AT*EDME?     are data menus enabled?
 *   AT*ESVM?     current voice mail server number?
 *   AT*ESVM=0
 *   AT+CKPD="eseee<see<s>>s"
 *   AT*ESVM=0
 *
 *   AT+CKPD="ss<91c91c9ss"     first Yes in WWW/Email mode
 *
 *   In SMS (others ?) < and > behave interestingly:
 *   You can "go right" with > at will, and "come back" with <,
 *   but not "step past left edge" with <. Eh?
 *
 *   (SMS/etc) mode reverts automatically back to nomode,
 *   after longish silent period. Nothing is emitted.
 *
 *   There is a pending state after SMS/etc, the board
 *   waits for something in addition to the "OK\r\n".
 *   It loses patience in a couple of seconds, then
 *   any entered and queued keys will be blurted out.
 */

#include "standard.h"

STATIC byte
*functions[] = {
	"\003" "e"                , /* ^C   No         */
	"\010" "c"                , /* ^H   <-         */
	"\012" "s"                , /* NL   Yes        */
	"\012" "ss<91c91c9ss"     , /* NL   first Yes in WWW/Email mode */
	"<"    "<"                , /* <    Left       */
	">"    ">"                , /* >    Right      */

	"\200" "91c91c911111110c" , /* ?    WWW        */
	"\201" "#11111110c"       , /* ?    Attachment */
	"\202" "8666*1111111*0c"  , /* ?    Email      spurious SMS first */
	"\203" "eseee<see<<s>>ss1", /* ?    SMS        */
	"\204" "eseee<see<s>>s"   , /* ?    Phonebook  */
},
*keys[] = {                     /* _ marks non-existent keys */
	"+&@/_%$____0__\205\206",  /* 2 last: Psi and Omega */
	" -?!,.:_\"'___()1",
	"abc2",
	"def3",
	"ghi4",
	"jkl5",
	"mno6",
	"pqrs7\207\210",           /* 2 last: Pi and Sigma */
	"tuv8",
	"wxyz9",
},
*shorthand[] = {
	"",             /* 0200 WWW */
	"",             /* 0201 Attachment */
	"",             /* 0202 Email */
	"",             /* 0203 SMS */
	"\003sendnow",  /* 0204 Phonebook */
	"",             /* 0205 Psi */
	"\003omnidf ",  /* 0206 Omega */
	"\003profile ", /* 0207 Pi */
	"",             /* 0210 Sigma */
};

/* return ascii character or control character or 0.
 * 'str' trashed. */

static byte
characterize(byte *str)
{
	byte *end, **v;

	/* only AT+CKPD="foo"[,20] contains double quotes */

	if ((str = index(str, '"')) == NULL
	 || (end = index(++str, '"')) <= str)  /* includes NULL */
		return (0);

	*end++ = 0;               /* str now tidy, *end is \0 or comma */

	if (!strcmp(end, ",25"))
		return (0);           /* superfluous AT+CKPD="c",25 ignored */

	v = functions;
	do {
		if (!strcmp(*v + 1, str))
			return (**v);
	} while (++v < __ENDOF(functions));

	if (*end)
		return (str[0]);      /* digits have ,20 */

	if (str[1] == 'c')
		str += 2;             /* 1c333/0c1 oddities equal 333/1 */

	if (*str == '*')
		return ('_');           /* shift space */

	if (*str == '#')
		return (str[1] ? '*' : '#');   /* ## and # translate to * and # */

	if (__isdigit(*str))
		return (keys[*str - '0'][strlen(str + 1)]);

	return (0);
}

/* chatboard_init(); needs to be called once */

extern byte chatboard_cnt, chatboard_buf[];

/* control characters returned are: 0 for no data,
 * ^C NL BS for No/Yes/Backspace, and
 * 0200 ... 0210 for Pi, Omega etc */

byte
chatboard_getc()
{
	byte c = 0;
	byte n = chatboard_cnt;

	if (n && chatboard_buf[--n] == '\r') {

		chatboard_buf[n] = 0;

		/* after the CR, chatboard waits us to ack 'OK' */

		if (debug) {
			print_string(chatboard_buf);
			eol();
		}
		c = characterize(chatboard_buf);

		chatboard_cnt = 0;
		chatboard_OK();
	}
	return (c);
}

void
process_chatboard()
{
	byte c = chatboard_getc();

	if (c) {
		/* "shorthand keys" are 0200 ... */

		if ((c - 0200) < __SIZEOF(shorthand)) {
			byte *p = shorthand[c - 0200];

			while (c = *p++)
				keyboard(c);
		} else
			keyboard(c);
	}
}

#if 0

void
chatboard_debug()
{
	static byte cb_rem, fill;

	while (cb_rem != chatboard_cnt) {
		byte c = chatboard_buf[cb_rem++];

		if (c == '\r') {
			chatboard_OK();
			eol();
		} else if (__isprint(c))
			putchar(c);
		else {
			print_string("\\x");
			print_byte(c);
		}
	}
}
#endif

