
/*
 * sigh.
 *
 * detecting wrap/ordering of incoming message ids... hurts the head.
 * keep last 6 different rcv_ids ?
 * requiring, or just utilizing when handed, TIMESTAMPS !? hmm...
 */

#include "standard.h"

#define AXALEN    7
#define TIMEOUT 100  /* 33, 66, 99 */

struct msg {
	struct msg *next;
	byte len;
	byte data[67];          /* aligns to even */
};

static struct msg *msg_free, msg_space[20];

static
struct xch {
	struct xch *next;
	byte addr[AXALEN];
	byte rcv_id[5];         /* aligns to even */
	byte snd_id[2];
	struct msg *pending;
	word mark, timo;
}
	*xch_list, xch_space[5];

static void
free_one(struct msg **list)
{
	struct msg *msg;

	if (msg = *list) {
		*list = msg->next;
		msg->next = msg_free;
		msg_free = msg;
	}
}

/* XXX just 00 ... 99, 00, 01 instead ? or hex. */

static void
bump(byte id[2])
{
	byte carry = 2;

	do {
		switch (id[1]) {
		default:
			id[1]++;
			return;
		case '9':
			id[1] = 'A';
			return;
		case 'Z':
			id[1] = 'a';
			return;

		case 0:
		case 'z':
			id[1] = '0';
		}
		--id;
	} while (--carry);
}

static void
discard_sendq(struct xch *x)
{
	while (x->pending)
		free_one(&x->pending);

	bump(x->snd_id);
}

static struct xch *
alloc_xch(byte *addr)
{
	struct xch **pp, **opp, *x;

	/* locate by addr, or take the most stale */

	for (pp = &xch_list; x = *pp; opp = pp, pp = &x->next)
		if (!axacmp(addr, x->addr))
			goto lift;

	pp = opp;
	x = *pp;

	/* make way */

	discard_sendq(x);
	x->rcv_id[0] = 0;
	axacpy(x->addr, addr);
lift:
	*pp = x->next;
	x->next = xch_list;
	xch_list = x;
	return (x);
}

static void
send_from(struct xch *x)
{
	struct txq *pkt;
	byte *p;

	if (p = aprs_messaging_prologue(x->addr, &pkt)) {

		struct msg *msg = x->pending;

		bcopy(msg->data, p, msg->len);
		p += msg->len;

		/* XXX stop sending replyack at some point ? */

		p[0] = '{';
		p[1] = x->snd_id[0];
		p[2] = x->snd_id[1];
		p += 3;

		if (x->rcv_id[0]) {
			byte *rack = x->rcv_id;
			byte n = 1+5;

			*p++ = '}';
			while (--n && isalnum(*rack))
				*p++ = *rack++;
		}

		x->timo += 30 + bits_of_random(3);
		x->mark = bolt();

		txq_enter_p(pkt, p);
	}
	/* else it re-timeouts sooner, as mark/timo is not changed XXX += 5 ? */
}

/*
 *  ...
 *  msg_exchange(addr, msgid, ack);
 *  msg = alloc_msg(addr);
 *  msg->data = ...
 *  msg->len = ...
 *  send_msg(msg);
 *  ...
 */

struct msg *
alloc_msg(byte *addr)
{
	struct xch *x = alloc_xch(addr);
	struct msg *msg;

	if (msg = msg_free) {
		msg_free = msg->next;
		msg->next = (void *) x;
	}
	return (msg);
}

void
send_msg(struct msg *msg)
{
	struct xch *x = (void *) msg->next;
	bool idle = !x->pending;

	msg->next = NULL;
	link_list_append(&x->pending, msg);

	if (idle) {
		x->timo = 0;
		send_from(x);
	}
}

/*
 *  ackXX     our snd_id XX is acked
 *  ackXX}ZZ  ditto, peer does not understand {} ?
 *  {Y}XX     incoming msgid Y, ack our snd_id XX
 *  {Y        incoming msgid Y. just store Y. no replyack
 *  {YY}      ditto
 *
 *  arguments 'msgid' and 'ack' appear as in the incoming message.
 */
void
msg_exchange(byte *addr, byte *msgid, byte *ack)
{
	struct xch *x = alloc_xch(addr);

	/* XXX test for duplicate (out-of-order detection is harder) */

	if (msgid)
		bcopy(msgid, x->rcv_id, sizeof(x->rcv_id));

	if (ack && x->pending
	 && ack[0] == x->snd_id[0]
	 && ack[1] == x->snd_id[1]) {

		free_one(&x->pending);
		bump(x->snd_id);
		if (x->pending) {
			x->timo = 0;
			send_from(x);
		}
	}
}

void
timeout_messages()
{
	struct xch *x;

	/* XXX send ackY coupla times if nothing else ? */

	for (x = xch_list; x; x = x->next)
		if (x->pending && bolt() - x->mark > x->timo) {
			if (x->timo < TIMEOUT)
				send_from(x);
			else
				discard_sendq(x); /* however, left at top of xch_list */
		}
}

void
init_message_exchange()
{
	xch_list = NULL;
	link_list_use(sizeof(xch_space), sizeof(*xch_list), xch_space, &xch_list);

	msg_free = NULL;
	link_list_use(sizeof(msg_space), sizeof(*msg_free), msg_space, &msg_free);
}

