/*
 *  Radio Shack WX200
 *  WMR918
 */

#include "standard.h"
#include "wx.h"

/* holds any of the messages, cksum not included */

static byte msg[34];


static word
bcd2(byte bcd)
{
	return (10 * (bcd >> 4) + (bcd & 0xF));
}

static word
bcd4(word bcd)
{
	return (100 * bcd2(bcd >> 8) + bcd2(bcd));
}

/* pick interesting stuff out from msg[] */

static void
rsw_msg()
{
	int i;
	word w;

	switch (msg[0]) {

	default:
		doh("rsw");

	/* WMR918 */

	case 0x00: /* wind */
		break;

	case 0x01: /* rain */
		break;

	case 0x02: /* extra sensors */
		break;

	case 0x03: /* temp, hum */
		break;

	case 0x05: /* pressure, maybe */
		break;

	case 0x06: /* maybe pressure */
		break;

	case 0x0E: /* seqno */
	case 0x0F: /* hourly status */
		break;

	/* WX200 */

	/*  8F to BF every 10 seconds, CF every 5. */

	case 0x8F:
		relative_humidity = bcd2(msg[20]); /* Humid Outdoor % */
		have_hygrometer = TRUE;
		break;

	case 0x9F:
		w = ld_word_intel(&msg[16]); /* Temp Outdoor 0.1 C */
		i = bcd4(w & 0x7FF);
		i = muldiv(i, , );
		if (w & 0x800)
			i = -i;
		temperature_celsius    = ;
		temperature_fahrenheit = ;
		have_thermometer = TRUE;
		break;

	case 0xAF:
		w = ld_word_intel(&msg[1]);    /* Barom Local mbar */
		barometric_pressure = bcd4(w);
		have_barometer = TRUE;
		break;

	case 0xBF:
		w = ld_word_intel(&msg[1]) & 0xFFF;  /* Rain Rate mm/hr */
		rain_rate_mmhr = bcd4(w);
		break;

	case 0xCF:
		w = ld_word_intel(&msg[1]) & 0xFFF;  /* Gust 0.2 m/s */
		wind_gust_ms = muldiv(w, , );

		w = ld_word_intel(&msg[4]) & 0xFFF;  /* Wind 0.1 m/s */
		wind_avg_ms = muldiv(w, , );

		wind_dir = bcd4(((msg[5] & 0xF0) << 4) | msg[6]); /* Dir deg */
		break;
	}
}

/* averages to less than 17 bytes per second, with WX200 */

void
rsw_input(byte c)
{
	static byte
		*insert,
		syncix, synced, remain, sum;

	if (synced) {
		if (remain == 0) {
			switch (c) {

			/* cksum byte will be included in remain */

			/* WMR918 */

			case 0xFF:
				return; /* ignore WMR918 FF FF extra header */

			case 0x00: remain =  9; break;
			case 0x01: remain = 14; break;
			case 0x02: remain =  7; break;
			case 0x03: remain =  7; break;
			case 0x05: remain = 11; break;
			case 0x06: remain = 12; break;
			case 0x0E: remain =  3; break;
			case 0x0F: remain =  7; break;

			/* WX200 */

			case 0x8F: remain = 35; break;
			case 0x9F: remain = 34; break;
			case 0xAF: remain = 31; break;
			case 0xBF: remain = 14; break;
			case 0xCF: remain = 27; break;

			default:
				goto sync_lost;
			}
			sum = 0;
			insert = msg;
		}
		if (--remain) {
			sum += c;
			*insert++ = c;
			return;
		}
		if (c == sum) {
			rsw_msg();
			return;
		}
		/* look for resync only after msg[] is filled once */

	sync_lost:
		synced = FALSE;
		sum = 0;
		remain = sizeof(msg);
	}

	if (remain)
		remain--;
	else {
		if (msg[syncix] == 0x8F && c == sum) {
			synced = TRUE;
			return;
		}
		sum -= msg[syncix];
	}
	sum += c;
	msg[syncix++] = c;

	if (syncix == sizeof(msg))
		syncix = 0;
}

void
rsw_init()
{
	/* 9600N8 */

	(9600);
}


