
#include <sys/types.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>

typedef unsigned char  byte;
typedef unsigned short word;
typedef unsigned long  dword;

static dword
EMUL(word e, word d)
{
	return ((dword) e * d);
}

static word
EDIV(dword ed, word div, word *rem)
{
	if (rem)
		*rem = ed % div;
	return (ed / div);
}

static dword
lnegate(dword ed)
{
	return (-ed);
}

static byte sintab[] = {
   0,  4,  8, 13, 17, 22, 26, 31, 35, 40,
  44, 48, 53, 57, 61, 66, 70, 74, 79, 83,
  87, 91, 95,100,104,108,112,116,120,124,
 127,131,135,139,143,146,150,154,157,161,
 164,167,171,174,177,181,184,187,190,193,
 196,198,201,204,207,209,212,214,217,219,
 221,223,226,228,230,232,233,235,237,238,
 240,242,243,244,246,247,248,249,250,251,
 252,252,253,254,254,255,255,255,255,255,
 255,255,255,255,255,255,254,254,253,252,
 252,251,250,249,248,247,246,244,243,242,
 240,238,237,235,233,232,230,228,226,223,
 221,219,217,214,212,209,207,204,201,198,
 196,193,190,187,184,181,177,174,171,167,
 164,161,157,154,150,146,143,139,135,131,
 127,124,120,116,112,108,104,100, 95, 91,
  87, 83, 79, 74, 70, 66, 61, 57, 53, 48,
  44, 40, 35, 31, 26, 22, 17, 13,  8,  4,
   0
};

/* partial arctangent, 45 degrees, 64 slots */

static word
atantab(word a, word b)
{
	static byte t[] = {
	   0,  1,  2,  3,  4,  4,  5,  6,
	   7,  8,  9, 10, 11, 11, 12, 13,
	  14, 15, 16, 17, 17, 18, 19, 20,
	  21, 21, 22, 23, 24, 24, 25, 26,
	  27, 27, 28, 29, 29, 30, 31, 31,
	  32, 33, 33, 34, 35, 35, 36, 36,
	  37, 37, 38, 39, 39, 40, 40, 41,
	  41, 42, 42, 43, 43, 44, 44, 45,
	  45
	};

	if (b)
		b = t[EDIV(EMUL(64, a), b, NULL)];

	return (b);
}

static long accu_ns, accu_ew;

static void
windavg_add(word rate, word deg)
{
	if (rate) {
		/* east/west component */

		if (deg < 180)
			accu_ew += EMUL(rate, sintab[deg]);
		else
			accu_ew -= EMUL(rate, sintab[deg - 180]);

		/* north/south component */

		deg += 90;
		if (deg >= 360)
			deg -= 360;

		if (deg < 180)
			accu_ns += EMUL(rate, sintab[deg]);
		else
			accu_ns -= EMUL(rate, sintab[deg - 180]);
	}
}

static word
windavg_result()
{
	word deg, q = 0;

	printf("%4.1f ", 180 * atan2(accu_ew, accu_ns) / M_PI);

	if (accu_ew < 0) {
		accu_ew = lnegate(accu_ew);
		q |= 2;
	}
	if (accu_ns < 0) {
		accu_ns = lnegate(accu_ns);
		q |= 1;
	}
	while (accu_ew > 0xFFFFu || accu_ns > 0xFFFFu) {
		accu_ew >>= 1;
		accu_ns >>= 1;
	}
	if (accu_ns > accu_ew)
		deg = atantab(accu_ew, accu_ns);
	else
		deg = 90 - atantab(accu_ns, accu_ew);

	accu_ew = accu_ns = 0;

	switch (q) {
	case 1:
		deg = 180 - deg;
		break;
	case 2:
		deg = 360 - deg;
		break;
	case 3:
		deg += 180;
		break;
	}
	printf("%3d\n", deg);

	return (deg);
}

main()
{
	word i;

	for (i = 0; i < 360; ++i) {
		windavg_add(100, i);
		windavg_result();
	}

	windavg_add(10, 80);
	windavg_result();

	windavg_add(10, 100);
	windavg_result();

	windavg_add(10, 190);
	windavg_result();

	windavg_add(10, 280);
	windavg_result();

	windavg_add(10, 320);
	windavg_add(10, 310);
	windavg_add(10, 330);
	windavg_add(10, 320);
	windavg_add(10, 330);
	windavg_add(10, 340);
	windavg_add(10, 340);
	windavg_add(10, 340);
	windavg_add(10, 340);
	windavg_add(10, 340);
}

